#ifndef DoublePrn_h
#define DoublePrn_h

/******************* random.h *********************/
/* modified ranstuff.c of MILC code */
/*  c++ language random number objects for parallel processors */
/*  exclusive or of feedback shift register and integer congruence
    generator.  Use a different multiplier on each generator, and make sure
    that fsr is initialized differently on each generator.  */

class DoublePrn
{
private:
   /* We assume long is at least 32 bits */
    unsigned long r0,r1,r2,r3,r4,r5,r6;
    unsigned long multiplier,addend,ic_state;
    float scale;

    static DoublePrn* prn;

public:

    DoublePrn(int seed, int index=0);
    ~DoublePrn(){ if(prn !=0) delete prn;}

    static DoublePrn* getRand() {return prn;}
    static void setRand(int seed) {
	prn = new DoublePrn(seed);
    }

   /* uniform deviate between 0 and 1 */
    float myrand(){
    register int t,s;
    t = ( ((r5 >> 7) | (r6 << 17)) ^ ((r4 >> 1) | (r5 << 23)) ) & 0xffffff;
    r6 = r5; r5 = r4; r4 = r3; r3 = r2; r2 = r1; r1 = r0; r0 = t;
    s = ic_state * multiplier + addend;
    ic_state = s;
    return( scale*(t ^ ((s>>8)&0xffffff)) );
  }

   /* normal deviate P(x) = exp(-x*x) */
    float gaussian_rand_no(){
    static int iset=0;
    static float gset;
    float fac,r,v1,v2;

    if  (iset == 0) {
	do {
	    v1=2.0*myrand()-1.0;
	    v2=2.0*myrand()-1.0;
	    r=v1*v1+v2*v2;
	} while (r >= 1.0);
	fac=sqrt( -log((double)r)/(double)r);
	gset=v1*fac;
	iset=1;
	return v2*fac;
    } else {
	iset=0;
	return gset;
    }
   }


/******************  rand_ahmat.c  (in su3.a) ***************************
*									*
* void random_anti_hermitian( anti_hermitmat *mat_antihermit, passthru *prn_pt)*
* Creates gaussian random anti-hermitian matrices			*
* Normalization is < |m01|^2 > = 1, or < m01.real*m01.real > = 1/2	*
* The argument "prn_pt" is a pointer to be passed to gaussian_rand_no() *
* RS6000 may choke on void *						*
*/

    //SU3Adjoint random_anti_hermitian()
    //SU3Adjoint  grand_SU3Adjoint();
    //su2_adjoint grand_su2_adjoint();
    //SU3Adjoint  grand_SU3AdjointSU2();


}; // struct DoublePrn

#endif



