#include <string>
#include <iostream>
#include <iomanip>
#include <cmath>
#include <limits>
#include "nr.h"
using namespace std;



//FT 1d
void FT1d(double* pArr,int nx,int sign)
{
  int l;
  double norm;

  Vec_DP divalpha(nx*2);
  norm=nx;
  Vec_INT nn(1);
  nn[0]=nx;

  for (int k=0;k<nx;k++)
    {
	l=k;
	l=(l<<1);  //multiplies l by 2
	divalpha[l]=pArr[l];
	l++;
	divalpha[l]=pArr[l];
      }

  NR::fourn(divalpha,nn,sign);

  for (int k=0;k<nx;k++)
    {
      l=k;
      l=(l<<1);  //multiplies l by 2
      pArr[l]=divalpha[l];
      if (sign==-1)
	pArr[l]/=norm;
      l++;
      pArr[l]=divalpha[l];
      if (sign==-1)
	pArr[l]/=norm;
    }
  //done
}


//FT in 2d

void FT(double* pArr,int nx, int ny,int sign)
{
  int l;
  double norm;
  //cout << "came here" <<endl;

  //check wether nx=ny
  if (nx!=ny) 
    {
      cout << "FFT Fatal: Cannot handle nx!=ny" << endl;
      exit(1);
    }

  Vec_DP divalpha(nx*ny*2);
  norm=nx*ny;
  Vec_INT nn(2);
  nn[0]=nx;
  nn[1]=ny;

  for (int j=0;j<nx;j++)
    for (int k=0;k<ny;k++)
      {
	l=k+j*nx;
	l=(l<<1);  //multiplies l by 2
	//real part, padding rest with zeros
	divalpha[l]=pArr[l];
	l++;
	divalpha[l]=pArr[l];
      }

  NR::fourn(divalpha,nn,sign);

  for (int j=0;j<nx;j++)
    for (int k=0;k<ny;k++)
      {
	l=k+j*nx;
	l=(l<<1);  //multiplies l by 2
	pArr[l]=divalpha[l];
	if (sign==-1)
	  pArr[l]/=norm;
	l++;
	pArr[l]=divalpha[l];
	if (sign==-1)
	  pArr[l]/=norm;
      }
  //done
}

//FT in 3d

void FT3d(double* pArr,int nx, int ny, int nz, int sign)
{
  int l;
  double norm;
  //cout << "came here" <<endl;

  //check wether nx=ny
  if (nx!=ny) 
    {
      cout << "FFT Fatal: Cannot handle nx!=ny" << endl;
      exit(1);
    }

  Vec_DP divalpha(nx*ny*nz*2);
  norm=nx*ny*nz;
  Vec_INT nn(3);
  nn[0]=nx;
  nn[1]=ny;
  nn[2]=nz;

  for (int j=0;j<nx;j++)
    for (int k=0;k<ny;k++)
      for (int m=0;m<nz;m++)
      {
	l=m+k*nz+j*ny*nz;
	l=(l<<1);  //multiplies l by 2

	divalpha[l]=pArr[l];
	l++;
	divalpha[l]=pArr[l];
      }

  NR::fourn(divalpha,nn,sign);

  for (int j=0;j<nx;j++)
    for (int k=0;k<ny;k++)
      for (int m=0;m<nz;m++)
	{
	  l=m+k*nz+j*ny*nz;
	  l=(l<<1);  //multiplies l by 2
	  pArr[l]=divalpha[l];
	  if (sign==-1)
	    pArr[l]/=norm;
	  l++;
	  pArr[l]=divalpha[l];
	  if (sign==-1)
	    pArr[l]/=norm;
	}
  //done
}


//transforms a 2d real data array to a complex NR type

void transform(double* pArr,int rnx, int rny)
{
  int l;
  int nx,ny,powx,powy;
  double norm;
  //cout << "came here" <<endl;

  //check wether nx=ny
  if (nx!=ny) cout << "FFT Fatal: Cannot handle nx!=ny" << endl;


  //cout << "Just to check: " << pArr[10] <<endl;

  //check whether rnx and rny are powers of 2:

  powx=0;
  while ((rnx>>powx)>1)
    powx++;

  //cout << "power nx:" << powx;

  powy=0;
  while ((rny>>powy)>1)
    powy++;

  //cout << "power ny:" << powy << endl;
  
  norm=rnx/double(1<<powx);

  if (norm==1) 
    {
      nx=rnx;
    }
  else
    {
      cout << "FFT Fatal: nx not a power of two. Exiting" << endl;
      exit(1);
      nx=((1<<powx)<<1);
    }


  norm=rny/double(1<<powy);
  if (norm==1) 
    {
      ny=rny;
    }
  else 
    {
      cout << "FFT Fatal: ny not a power of two. Exiting" << endl;
      exit(1);
      ny=((1<<powy)<<1);
    }
  //cout << "nx, ny:" << nx << " , " << ny << endl;


  Vec_DP data(nx*ny*2);
  Vec_DP data2(nx*ny*2);
  norm=nx*ny;
  Vec_INT nn(2);
  nn[0]=nx;
  nn[1]=ny;
  for (int j=0;j<nx;j++)
    for (int k=0;k<ny;k++)
      {
	l=k+j*nx;
	l=(l<<1);  //multiplies l by 2
	//real part, padding rest with zeros
	//if ((k<rny)&&(j<rnx))
	data[l]=pArr[k+j*rnx];
  //else 
  //  { data[l]=0; }
	//cout << l << " : " << data[l] << endl;
	//imaginary part
	l++;
	data[l]=0;
	//cout << l << " : " << data[l] << endl;
      }
  NR::fourn(data,nn,1);
  //do solution of Poisson's equation in FS here

  //first check if there is really no zero mode:
  if (data[0]>1.e-10)
    cout << "FFT Fatal: Zero mode non-vanishing!" << endl;

  //cout << "data[2] = " << data[2] << " data[3] = " << data[3] << endl;
  //cout << "data[4] = " << data[4] << " data[5] = " << data[5] << endl;
  //cout << "data[6] = " << data[6] << " data[7] = " << data[7] << endl;
  //cout << "data[8] = " << data[8] << " data[9] = " << data[9] << endl;

  //double dataav=0;

  for (int j=0;j<nx;j++)
    for (int k=0;k<ny;k++)
      {
	l=k+j*nx;
	l=(l<<1);
	//dataav+=data[l]*data[l]+data[l+1]*data[l+1];
	if (l!=0)
	  {
	    data2[l]=data[l]/2/(cos(6.28318530717958647692529*j/nx)+
				cos(6.28318530717958647692529*k/ny)-2);
	    l++;
	    data2[l]=data[l]/2/(cos(6.28318530717958647692529*j/nx)+
				cos(6.28318530717958647692529*k/ny)-2);
	  }
	if (l==0)
	  {
	    data2[0]=0;
	    data2[1]=0;
	  }
      }
  //cout << "FFav rho = " << dataav/norm << endl;


  //cout << "adata[2] = " << data2[2] << " data[3] = " << data2[3] << endl;
  //cout << "adata[4] = " << data2[4] << " data[5] = " << data2[5] << endl;
  //cout << "adata[6] = " << data2[6] << " data[7] = " << data2[7] << endl;
  //cout << "adata[8] = " << data2[8] << " data[9] = " << data2[9] << endl;
  //cout << "adata[16] = " << data2[16] << " data[17] = " << data2[17] << endl;
  //cout << "adata[34] = " << data2[34] << " data[35] = " << data2[35] << endl;


  /*
  for (int j=0;j<nx;j++)
    for (int k=0;k<ny;k++)
      {
	l=k+j*nx;
	l=(l<<1);
	data2[l]=data[l];
	l++;
	data2[l]=data[l];
      }
  NR::fourn(data,nn,-1);
  */


  NR::fourn(data2,nn,-1);
  //give it back in array

  for (int j=0;j<nx;j++)
    for (int k=0;k<ny;k++)
      {
	l=k+j*nx;
	l=(l<<1);  //multiplies l by 2
	//cout << data2[l]/norm << endl;
	//real part
	//if ((k<rny)&&(j<rnx))
	//{ 
	    pArr[k+j*rnx]=data2[l]/norm; 
	    //check any weird signals
	    if ((data2[l+1]/data2[l]>1.e-10)&&(data2[l]!=0))
	      {
		cout << "FFT Warning: got nonzero imaginary part: ";
		cout << data2[l+1];
		cout << " vs real: ";
		cout << data2[l];
		cout << endl;
	      }
	    //}
	l++;

	//cout << "C:" << data2[l] << " vs " << data[l] << endl;
      }
  
  //cout << "Just to check: " << pArr[10] <<endl;
  cout << "FFT done" << endl;
}

/*
int main(void)
{
  int l;
  
  int nx,ny;
  nx=31;
  ny=31;

  double myArr[nx*ny];
  myArr[1]=1;



  for (int j=0;j<nx;j++)
    for (int k=0;k<ny;k++)
      {
	l=k+j*nx;
	myArr[l]=k+j*nx;
      }
  cout << "at 42: " << myArr[42] <<endl;
  transform(myArr,nx,ny);
  cout << "check again:" << myArr[42] <<endl;
}

*/
