/* Implements the Polar form of the Box-Muller Transformation
   to generate gaussian random numbers.  Modified to work with
   rand()
   
   (c) Copyright 1994, Everett F. Carter Jr.
       Permission is granted by the author to use
       this software for any application provided this
       copyright notice is preserved.
*/

#include <math.h>
#include <stdlib.h>



double box_muller(double m, double s)	/* normal random variate generator */
{				        /* mean m, standard deviation s */
	double x1, x2, w, y1;
	static double y2;
	static int use_last = 0;

	if (use_last)		        /* use value from previous call */
	{
		y1 = y2;
		use_last = 0;
	}
	else
	{
		do {
			x1 = 2.0 * rand()/RAND_MAX - 1.0;
			x2 = 2.0 * rand()/RAND_MAX - 1.0;
			w = x1 * x1 + x2 * x2;
		} while ( w >= 1.0 );

		w = sqrt( (-2.0 * log( w ) ) / w );
		y1 = x1 * w;
		y2 = x2 * w;
		use_last = 1;
	}

	return( m + y1 * s );
}

// generates gaussian random number with standard deviation s and mean 0
double randGauss(double s)
{
	return box_muller(0,s);
}
